#include "mv.hh"
#include <assert.h> // for assert
#include <cmath>
#include <stdlib.h> // for rand() function

// initialize structure to a vector of size 0
Vec::Vec() {
  cout << "Call constructor for Vec\n" ;
  _size  = 0 ;
  values = nullptr ;
}

Vec::Vec( int _sz ) : _size(_sz) {
  cout << "Call constructor for Vec with size = " << _size << "\n" ;
  values = new double[_size] ; // allocate memory for the vector
}

// copy constructor
Vec::Vec( Vec const & a ) {
  cout << "Vec: Call copy constructor\n" ;
  allocate(a.size()) ;
  for ( int i = 0 ; i < _size ; ++i ) values[i] = a(i) ;
}

// copy operator
Vec const & 
Vec::operator = ( Vec const & a ) {
  cout << "Vec: Call copy operator\n" ;
  allocate(a.size()) ;
  for ( int i = 0 ; i < _size ; ++i ) values[i] = a(i) ;
  return *this ;
}

Vec::~Vec( ) {
  cout << "Call detructor for Vec\n" ;
  // check if alreay de-allocated
  if ( values != nullptr ) delete [] values ;
  _size = 0 ;
}

// alocate a vector to teh size: size
void
Vec::allocate( int size ) {
  _size = size ;
  // check if alreay allocated
  if ( values != nullptr ) delete [] values ;
  values = new double[_size] ;
}

void
Vec::fill( double val ) {
  for ( int i = 0 ; i < _size ; ++i )
    values[i] = val ;
}

void
Vec::fill_random() {
  for ( int i = 0 ; i < _size ; ++i )
    values[i] = std::floor(100.0*((rand() - rand())/double(RAND_MAX))) ;
}

/*
 ___    _____  
|_ _|  / / _ \ 
 | |  / / | | |
 | | / /| |_| |
|___/_/  \___/ 

*/

ostream &
operator << ( ostream & stream, Vec const & vec ) {
  stream << vec(0) ;
  for ( int i = 1 ; i < vec.size() ; ++i )
    stream << " " << vec(i) ;
  stream << '\n' ;
  return stream ;
}

/*
    _    _            _               
   / \  | | __ _  ___| |__  _ __ __ _ 
  / _ \ | |/ _` |/ _ \ '_ \| '__/ _` |
 / ___ \| | (_| |  __/ |_) | | | (_| |
/_/   \_\_|\__, |\___|_.__/|_|  \__,_|
           |___/                      
*/

Vec
operator + ( Vec const & a, Vec const & b ) {
  assert( a.size() == b.size() ) ;
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i) + b(i) ;
  return res ;
}

Vec
operator - ( Vec const & a, Vec const & b ) {
  assert( a.size() == b.size() ) ;
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i) - b(i) ;
  return res ;
}

Vec
operator * ( Vec const & a, Vec const & b ) {
  assert( a.size() == b.size() ) ;
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i) * b(i) ;
  return res ;
}

Vec
operator / ( Vec const & a, Vec const & b ) {
  assert( a.size() == b.size() ) ;
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i) / b(i) ;
  return res ;
}

Vec
operator * ( double a, Vec const & b ) {
  Vec res(b.size()) ;
  for ( int i = 0 ; i < b.size() ; ++i ) res(i) = a*b(i) ;
  return res ;
}

Vec
operator * ( Vec const & a, double b ) {
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i)*b ;
  return res ;
}

Vec
operator / ( Vec const & a, double b ) {
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = a(i)/b ;
  return res ;
}

Vec
operator ^ ( Vec const & a, double b ) {
  Vec res(a.size()) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res(i) = std::pow(a(i),b) ;
  return res ;
}

double
dot( Vec const & a, Vec const & b ) {
  assert( a.size() == b.size() ) ;
  double res(0) ;
  for ( int i = 0 ; i < a.size() ; ++i ) res += a(i)*b(i) ;
  return res ;
}

double norm( Vec const & a ) { return sqrt(dot(a,a)) ; }
